@@ -1,9 +1,14 @@
-meldMC<-function(T1,T2, nullparm=NULL, parmtype=c("difference","oddsratio","ratio"),
+# meldMC= function that takes output from the Monte Carlo simulations of the 
+#         two confidence distribution random variables, say T1 and T2 (each length NMC)
+#         and defines the g function, creates the NMC Beta parameters, orders them
+#         and picks out the appropriate quantiles for the lower and/or upper limits
+meldMC<-function(T1,T2, nullparm=NULL, parmtype=c("difference","oddsratio","ratio","cdfratio"),
     conf.level=0.95, 
     alternative=c("two.sided","less","greater"),
     dname="",estimate1=NA, estimate2=NA){
 
     ptype<-match.arg(parmtype)
+    # create g function to go with the parameter type
     if (ptype=="difference"){
         g<-function(T1,T2){ T2-T1 }
         if (is.null(nullparm)) nullparm<-0
@@ -13,6 +18,12 @@ meldMC<-function(T1,T2, nullparm=NULL, parmtype=c("difference","oddsratio","rati
     } else if (ptype=="oddsratio"){
         g<-function(T1,T2){ T2*(1-T1)/( T1*(1-T2) ) }
         if (is.null(nullparm)) nullparm<-1
+    } else if (ptype=="cdfratio"){
+        # want a ratio of cumulative incidence curves
+        # to keep it increasing in T2 and decreasing in T1
+        # use....
+        g<-function(T1,T2){ (1-T1)/(1-T2)  }
+        if (is.null(nullparm)) nullparm<-1
     }
 
     lowerLimit<- g(1,0)
@@ -24,6 +35,7 @@ meldMC<-function(T1,T2, nullparm=NULL, parmtype=c("difference","oddsratio","rati
 
     if (alt=="two.sided"){
         dolo<-dohi<-TRUE
+        # for two-sided set alpha=half of the error
         alpha<-(1-conf.level)/2  
     } else if (alt=="less"){
         ## alt=less so lower interval is lowest possible, 
@@ -41,21 +53,28 @@ meldMC<-function(T1,T2, nullparm=NULL, parmtype=c("difference","oddsratio","rati
     } else stop("alternative must be 'two.sided', 'less', or 'greater' ")
 
     Beta<-g(T1,T2)
+    # create ordered version of Beta
     oBeta<-Beta[order(Beta)]
     NMC<-length(Beta)
 
     if (dolo){
+        # find k = index for the lower limit
         # see Efron and Tibshirani (1993) Introduction to the Bootstrap,  p. 160, bottom
         k<-floor((NMC+1)*alpha)
         if (k==0) warning("increase nmc, confidence limits may be anti-conservative")
+        # get lower limit
         lower<-oBeta[k]
+        # get associated one-sided p-value (associated with alternative=greater)
         pg<-length(Beta[Beta<=nullparm])/NMC
     }
     if (dohi){
+        # find k=index for the lower limit
         # see Efron and Tibshirani (1993) Introduction to the Bootstrap,  p. 160, bottom
         k<-floor((NMC+1)*alpha)
         if (k==0) warning("increase nmc, confidence limits may be anti-conservative")
+        # get upper limit, NMC=total, so (NMC+1-k):NMC are the indeces for the k largest values 
         upper<-oBeta[NMC+1-k]
+        # get associated one-sided p-value (associated with alternative=less)
         pl<-length(Beta[Beta>=nullparm])/NMC
 
     }
@@ -63,10 +82,14 @@ meldMC<-function(T1,T2, nullparm=NULL, parmtype=c("difference","oddsratio","rati
 
 
     # we do not get estimates from betaParms
+    # so if we want to output our beta estimate, we have to input the estimate from each group
+    # i.e., estimate1 and estimate2
     estimate<-g(estimate1,estimate2)
     names(estimate)<-ptype
 
     if (alt=="two.sided"){
+        # for two-sided p-value, we use the central method, 
+        # twice the minimum of the one-sided p-values (but not more than 1)
         p.value<- min(1,2*pl,2*pg)
     } else if (alt=="less"){
         p.value<- pl
@@ -83,8 +106,9 @@ meldMC<-function(T1,T2, nullparm=NULL, parmtype=c("difference","oddsratio","rati
     parm<-estimate2
     names(stat) <- "estimate 1"
     names(parm) <- "estimate 2"
-    names(nullparm)<-paste("Null",ptype)
+    names(nullparm)<-paste(ptype)
 
+    # create htest object so it prints nicely
     structure(list(statistic = stat, parameter = parm, 
         p.value = p.value, 
         conf.int = ci, estimate = estimate, null.value = nullparm, 
@@ -96,14 +120,17 @@ meldMC<-function(T1,T2, nullparm=NULL, parmtype=c("difference","oddsratio","rati
 
 
 
-
-
-
-
+# betaMeldTest is a function to run Meld tests
+# given lists of beta parameters from each group,
+# betaParms1 and betaParms2, where the beta parameters 
+# represent lower (alower and blower) and upper (aupper and bupper)
+# confidence distributions for the two groups.
+# betaMeldTest uses numeric integration, and does not 
+# have a midp=TRUE option
 betaMeldTest <-
 function(betaParms1,
     betaParms2,nullparm=NULL, 
-    parmtype=c("difference","oddsratio","ratio"),
+    parmtype=c("difference","oddsratio","ratio","cdfratio"),
     conf.level=0.95, conf.int=TRUE,
     alternative=c("two.sided","less","greater"),eps=10^-8,
     dname="",
@@ -153,8 +180,15 @@ function(betaParms1,
     } else if (ptype=="oddsratio"){
         g<-function(T1,T2){ T2*(1-T1)/( T1*(1-T2) ) }
         if (is.null(nullparm)) nullparm<-1
+    } else if (ptype=="cdfratio"){
+        # want a ratio of cumulative incidence curves
+        # to keep it increasing in T2 and decreasing in T1
+        # use....
+        g<-function(T1,T2){ (1-T1)/(1-T2)  }
+        if (is.null(nullparm)) nullparm<-1
     }
 
+
     lowerLimit<- g(1,0)
     upperLimit<-g(0,1)
 
@@ -167,6 +201,7 @@ function(betaParms1,
     # with g(p1,p2)= p2-p1  (difference)
     #   or g(p1,p2)= p2/p1  (ratio)
     #   or g(p1,p2)= p2(1-p1)/(p1(1-p2))  (oddsratio)
+    #   or g(p1,p2)= (1-p1)/(1-p2)  (cdfratio)
     #
     #  Want to test with D=nullparm
     #    greater:   H0: g(p1,p2) <= D
@@ -227,6 +262,23 @@ function(betaParms1,
         funcGreater<-function(t1,D){
             pbeta(t1*D/(1-t1+t1*D),aL2,bL2)*dbeta(t1,aU1,bU1)
         }
+    } else if (ptype=="cdfratio"){
+        # Pr[ g(T1,T2) >=D] = Pr[ (1-T1)/(1-T2) >= D]  
+        # =Pr[ 1-T1 >= (1-T2)D]   
+        # =Pr[ T1 <= 1 -(1-T2)D]  
+        # = \int F1(1-(1-t2)*D) f2(t2)  dt2
+        funcLess<-function(t2,D){
+            pbeta(1-(1-t2)*D,aL1,bL1)*dbeta(t2,aU2,bU2)
+        }
+        # Pr[ g(T1,T2) <=D] = Pr[ (1-T1)/(1-T2) <= D] 
+        # = Pr[ 1-T1 <= (1-T2)*D]
+        # = Pr[ (1-T1)/D  <= (1-T2)]
+        # = Pr[ -(1-T1)/D  >= -1+T2]
+        # = Pr[T2 <= 1 -(1-T1)/D]
+        # = \int F2( 1- (1-t1)/D) f1(t1)  dt1
+        funcGreater<-function(t1,D){
+            pbeta(1-(1-t1)/D,aL2,bL2)*dbeta(t1,aU1,bU1)
+        }
     }
 
 
@@ -243,6 +295,8 @@ function(betaParms1,
         ##    difference: W[t] = t + D 
         ##       ratio:   W[t] = t*D
         ##   odds ratio:  W[t] = t*D/(1-t+t*D)
+        ##       cdfratio:  W[t] = 1- (1-t)/D
+
         ##
         ## First, choose LowerInt=a and UpperInt=b so 
         ## that \int_a^b  dbeta1(t) dt = 1- eps/2 
@@ -258,6 +312,9 @@ function(betaParms1,
         } else if (ptype=="oddsratio"){
            # solve q = t*D/(1-t+t*D)   for t
            a2<- q/(delta+q-delta*q)
+        } else if (ptype=="cdfratio"){
+           # solve q = 1- (1-t)/D  for t
+            a2<- 1-delta*(1-q)
         }
         LowerInt<-max(a2,LowerInt)
 
@@ -286,6 +343,8 @@ function(betaParms1,
         ##    difference: W[t] = t - D 
         ##       ratio:   W[t] = t/D
         ##   odds ratio:  W[t] = t/(t+(1-t)*D)
+        ##      cdfratio:   W[t] = 1-(1-t)*D
+
         ##
         ## First, choose LowerInt=a and UpperInt=b so 
         ## that \int_a^b  dbeta2(t) dt = 1- eps/2 
@@ -302,6 +361,9 @@ function(betaParms1,
         } else if (ptype=="oddsratio"){
            # solve q = t/(t+(1-t)*D)   for t
            a2<- q*delta/(1-q+delta*q)
+        } else if (ptype=="cdfratio"){
+           # solve q= 1-(1-t)*D   for t
+            a2<- 1-(1-q)/delta
         }
         LowerInt<-max(a2,LowerInt)
         pout<-rep(0,length(delta))
@@ -344,6 +406,8 @@ function(betaParms1,
 
 
     # we do not get estimates from betaParms
+    # so if we want to output our beta estimate, we have to input the estimate from each group
+    # i.e., estimate1 and estimate2
     estimate<-g(estimate1,estimate2)
     names(estimate)<-ptype
 
@@ -354,24 +418,55 @@ function(betaParms1,
         if (aL2==0 & bU1>0){
             if (conf.int) lower<- g( qbeta(1-alpha,aU1,bU1), 0 )
             if (ptype=="difference"){ 
+                # Pr[ g(T1,0) = -T1 <= nullparm] 
+                # = Pr[ T1 >= -nullparm]
+                # = 1-Pr[ T1 <= -nullparm]
                 pg<- 1- pbeta(-nullparm,aU1,bU1)
-            } else pg<-1
+            } else if (ptype=="ratio"){
+                # Pr[ g(T1,0)=0/T1 <= nullparm]=I(0<=nullparm]=1
+                pg<-1
+            } else if (ptype=="oddsratio"){
+                # Pr[ g(T1,0)=(0*(1-T1))/(T1*(1-0)) <= nullparm]=I(0<=nullparm]=1
+                pg<-1
+            } else if (ptype=="cdfratio"){
+                # Pr[ g(T1,0)=(1-T1)/(1-0) <= nullparm]
+                # = Pr[ 1 - T1 <= nullparm] 
+                # = Pr[ T1 >= 1 - nullparm] 
+                # = 1 - Pr[ T1 <= 1-nullparm] 
+                pg<-1 - pbeta(1-nullparm, aU1, bU1)
+            }
         } else if (bU1==0 & aL2>0){
             if (conf.int) lower<- g( 1, qbeta(alpha,aL2,bL2) )
             if (ptype=="difference"){
                 # Jul 15 2014: Fix error
-                #pg<- 1-pbeta(1+nullparm,aL2,bL2)
+                # Error: pg<- 1-pbeta(1+nullparm,aL2,bL2)
+                # Here is proper motivation...
+                #  Pr[ g(1,T2)=T2-1 <= nullparm]
+                #  Pr[ T2 <= 1+ nullparm] 
                 pg<- pbeta(1+nullparm,aL2,bL2)
             } else if (ptype=="ratio"){
+                # Pr[g(1,T2)=T2 <=nullparm] 
                 pg<-pbeta(nullparm,aL2,bL2)
             } else if (ptype=="oddsratio"){
-                pg<-pbeta(nullparm/(1-nullparm),aL2,bL2)
+                # Error before Version 1.4.1
+                # Error: pg<-pbeta(nullparm/(1-nullparm),aL2,bL2)
+                # proper motivation....
+                # Pr[ g(1,T2) = (T2*(1-1))/(1*(1-T2)) = 0 <= nullparm]
+                # = I(0<= nullparm) = 1
+                pg<- 1
+            }  else if (ptype=="cdfratio"){
+                # Pr[g(1,T2)=(1-1)/(1-T2) = 0 <=nullparm] = I(0<=nullparm)=1 
+                pg<-1
             }
         } else if (aL2==0 & bU1==0){
             if (conf.int) lower<- g( 1, 0 )
             pg<-1
         } else {
             if (conf.int){ 
+                # get lower confidence limit by finding 
+                # the value of delta such that the associated
+                # one-sided p-value (for alternative=greater)
+                # equals alpha (recall alpha is the one-sided error)
                 rootfunc<-function(delta){
                     pGreater(delta)-alpha
                 }
@@ -413,23 +508,53 @@ function(betaParms1,
         } else if (aL1==0){
             if (conf.int) upper<- g(0,qbeta(1-alpha,aU2,bU2))
             if (ptype=="difference"){
+               # want Pr[ g(0,T2) >= nullparm] 
+               #  = Pr[ T2>= nullparm] = 1- Pr[T2<= nullparm]
                pl<-1-pbeta(nullparm, aU2,bU2)
             } else if (ptype=="ratio"){
+               # g(0,T2) = T2/0=Inf
+               # So Pr[ Inf >= nullparm] =1
                pl<- 1
             } else if (ptype=="oddsratio"){
+               # g(0,T2) = (T2*(1-0))/(0*(1-T2))=Inf
+               # So Pr[ Inf >= nullparm] =1
                pl<-1
+            } else if (ptype=="cdfratio"){
+               # we want Pr[ g(0,T2) = 1/(1-T2) >= nullparm]
+               #  Note:     1/(1-T2) >= nullparm
+               #  =>        1        >= nullparm*(1-T2)
+               #  => 1 + nullparm*T2 >= nullparm
+               #  =>              T2 >= (nullparm - 1)/nullparm              
+               pl<- 1- pbeta( (nullparm-1)/nullparm, aU2,bU2)
             }
         } else if (bU2==0){
             if (conf.int) upper<- g(qbeta(alpha,aL1,bL1), 1)
             if (ptype=="difference"){
-               pl<-pbeta(1+nullparm, aL1, bL1)
+               # NOTE: Error prior to Version 1.4.1
+               #       but not an error with nullparm=0.
+               # Error: pl<-pbeta(1+nullparm, aL1, bL1)
+               # proper motivation....
+               # Pr[ g(T1,1)=1-T1 >= nullparm]
+               # = Pr[ -1+T1 <= -nullparm]
+               # = Pr[ T1 <= 1-nullparm]
+               pl<- pbeta(1-nullparm, aL1, bL1)
             } else if (ptype=="ratio"){
+               # Pr[ g(T1,1)=1/T1 >=nullparm]
+               # Pr[ T1<= 1/nullparm]
                pl<-pbeta(1/nullparm,aL1,bL1)
             } else if (ptype=="oddsratio"){
+               # Pr[ g(T1,1)= (1-T1)/(T1*(1-1))=Inf >=nullparm]
+               pl<- 1
+            } else if (ptype=="cdfratio"){
+               # because g(T1,1) =(1-T1)/0 = Inf
                pl<- 1
             }
         } else {
             if (conf.int){
+              # get upper confidence limit by finding 
+              # the value of delta such that the associated
+              # one-sided p-value (for alternative=less)
+              # equals alpha (recall alpha is the one-sided error)
                 rootfunc<-function(delta){
                     pLess(delta)-alpha
                 }
@@ -477,8 +602,9 @@ function(betaParms1,
     parm<-estimate2
     names(stat) <- "estimate 1"
     names(parm) <- "estimate 2"
-    names(nullparm)<-paste("Null",ptype)
+    names(nullparm)<-paste(ptype)
 
+    # create htest object for output
     structure(list(statistic = stat, parameter = parm, 
         p.value = p.value, 
         conf.int = ci, estimate = estimate, 
@@ -488,12 +614,15 @@ function(betaParms1,
 
 }
 
-
-
+# betaMeldTestMidp.mc is a function to take the 
+# beta parameters from the confidence distributions,
+#    betaParms1=list of parameters from group 1
+#    betaParms2=list of parameters from group 2
+# and perform the mid-p inferences by Monte Carlo simulation 
 betaMeldTestMidp.mc <-
 function(betaParms1,
     betaParms2,nullparm=NULL, 
-    parmtype=c("difference","oddsratio","ratio"),
+    parmtype=c("difference","oddsratio","ratio","cdfratio"),
     conf.level=0.95, conf.int=TRUE,
     alternative=c("two.sided","less","greater"),
     dname="",
@@ -537,6 +666,9 @@ function(betaParms1,
     # rrbeta(n,a,b) is rbeta(n,a,b) but allow limits at 0 for parameters, so that 
     #      a=0, b>0 gives rep(0,n)  
     #      a>0, b=0 gives rep(1,n)
+    # 
+    # Note: in modern versions of R (after version 3.6.3 [and likely before that too])
+    # there is no need for the rrbeta function, since rbeta will do the same thing
     rrbeta<-function(n,a,b){
         if (length(a)>1 | length(b)>1) stop("rewrite rrbeta for vector parameters")
         if (a==0 & b==0){ 
@@ -600,17 +732,22 @@ bpcp2sampControl<-function(Delta=0, stype="km", eps=10^-8, nmc=10^6, method="mm.
 
 
 bpcp2samp<-function(time,status,group, testtime, 
-    parmtype=c("difference","oddsratio","ratio"),
+    parmtype=c("difference","oddsratio","ratio","cdfratio","one.minus.ratio","one.minus.cdfratio"),
     nullparm=NULL,
     alternative=c("two.sided","less","greater"), conf.level=0.95, 
     midp=FALSE, 
+    changeGroupOrder=FALSE,
     control=bpcp2sampControl()){
 
-    # so we get the same answer on the same data set
+    # so we get the same answer on the same data set, the default seed 
+    # has a fixed number
     # for simulations use control=bpcp2sampControl(seed=NULL)
     if (!is.null(control$seed)) set.seed(control$seed)
 
-    if (class(group)=="factor"){
+    # in case group is a factor with more than 2 levels, but only 2 levels are selected
+    # keep only the two levels with data
+    #if (class(group)=="factor"){
+    if (is(group,"factor")){
         # use order of levels, but only keep the levels with data 
         ug<- levels(group)
         ug<- ug[ug %in% group]
@@ -620,6 +757,12 @@ bpcp2samp<-function(time,status,group, testtime,
     if (length(ug)!=2) stop("group does not have 2 levels") 
     if ((length(group[group==ug[1]])<1) | (length(group[group==ug[2]])<1)){ 
         stop("should have at least one observation in each group") }
+    
+    if (changeGroupOrder){
+        ug<-ug[2:1]
+    }
+    
+    
     if (length(time)!=length(status) | length(time)!=length(group) ) stop("length of time, status and group should be the same")
     if (length(testtime)>1 | !is.numeric(testtime[1]) | testtime[1]<0) stop("testtime must be a vector of length 1")
     ## end of argument checking
@@ -630,7 +773,34 @@ bpcp2samp<-function(time,status,group, testtime,
     CL<-conf.level
     alt<-match.arg(alternative)
     
-    ptype<-match.arg(parmtype)
+    ptype<-PType<-match.arg(parmtype)
+    # ptype=types for calculations, 
+    # PType=original types
+    # both are the same, except for 'one.minus.ratio' and 'one.minus.cdfratio'
+    # For the one.minus types, we go through the calculations using the ratios
+    # after switching the one-sided alternative (if needed)
+    # and switching the Nullparm (if needed)
+    # and fix the output at the end by multiplying each by -1 and adding 1
+    switch.alt<-function(alt){
+      if (alt=="less"){
+        s.alt<- "greater"
+      } else if (alt=="greater"){
+        s.alt<- "less"
+      } else { 
+        s.alt<- "two.sided"
+      }
+      s.alt
+    }
+    if (PType=="one.minus.ratio"){
+      ptype<-"ratio"
+      alt<- switch.alt(alt)
+      if (!is.null(Nullparm)) Nullparm<- 1- Nullparm
+    } else if (PType=="one.minus.cdfratio"){
+      ptype<-"cdfratio"
+      alt<- switch.alt(alt)
+      if (!is.null(Nullparm)) Nullparm<- 1- Nullparm
+    }
+    
     if (ptype=="difference"){
       Dname<-paste("S(",testtime,";group=",ug[2],
                  ")-S(",testtime,";group=",ug[1],")",sep="")
@@ -641,8 +811,13 @@ bpcp2samp<-function(time,status,group, testtime,
       Dname<-paste("odds[S(",testtime,";group=",ug[2],
                 ")]/odds[S(",testtime,";group=",ug[1],
                  ")]",sep="")
-    }
-    
+    } else if (ptype=="cdfratio"){
+      #Dname<-paste("[1-S(",testtime,";group=",ug[1],")]/
+      #              [1-S(",testtime,";group=",ug[2],")]",sep="")
+      Dname<-paste("F(",testtime,";group=",ug[1],")/
+                    F(",testtime,";group=",ug[2],")",sep="")
+    } 
+
     # regardless of method, need estimate, so run 
     # fastest (nmc=0) for single sample
     I<-group==ug[1]
@@ -672,9 +847,15 @@ bpcp2samp<-function(time,status,group, testtime,
     list1<-getList(fit1,testtime)
     list2<-getList(fit2,testtime)
 
-    
+
     if (control$method=="mm.mc"){  
+      # mm.mc = method of moments for each group & 
+      #         Monte Carlo for melding the two groups together 
+      #         (but actually, when midp=FALSE use numeric integration
+      #         for the second part)
+      # mm= get beta parameters from the fit1 and fit2
       if (midp){
+        # Monte Carlo
         testout<-betaMeldTestMidp.mc(betaParms1=list1$betaParms,
                               betaParms2=list2$betaParms,
                               nullparm=Nullparm,
@@ -686,6 +867,7 @@ bpcp2samp<-function(time,status,group, testtime,
                               estimate2=list2$estimate,
                               nmc=control$nmc)  
       } else {
+        # use numeric integration
         testout<-betaMeldTest(betaParms1=list1$betaParms,
                               betaParms2=list2$betaParms,
                               nullparm=Nullparm,
@@ -700,7 +882,10 @@ bpcp2samp<-function(time,status,group, testtime,
     } else if (control$method=="mc.mc"){
         if (midp){
             I<-group==ug[1] 
+            # kmgw.calc is a fast function for getting Kalpan-Meier estimates
             x1<-kmgw.calc(time[I],status[I],keepCens=TRUE)
+            # bpcp.mc is a fast function for getting bpcp CIs at t=testtime
+            # using Monte Carlo methods
             mc1<-bpcp.mc(x1,nmc=control$nmc,
                     testtime=testtime,DELTA=control$Delta,
                     midp=TRUE)
@@ -709,6 +894,10 @@ bpcp2samp<-function(time,status,group, testtime,
             mc2<-bpcp.mc(x2,nmc=control$nmc,
                  testtime=testtime,DELTA=control$Delta,
                  midp=TRUE)
+            # meldMC inputs the Monte Carlo vectors of lower and upper 
+            # confidence distribution random variables (CDRVs), 
+            # and outputs the confidence limits
+            # midp uses both lower and upper CDRVs
             testout<-meldMC(c(mc1$Smc$Slo,mc1$Smc$Shi), 
                             c(mc2$Smc$Slo,mc2$Smc$Shi),
                               nullparm=Nullparm,
@@ -749,6 +938,8 @@ bpcp2samp<-function(time,status,group, testtime,
             } else stop("alternative must be 'two.sided', 'less', or 'greater' ")
 
             if (dolo){
+                # Shi are upper Confidence distribution RVs
+                # Slo are lower CDRVs
                 # take Shi from group 1, and Slo from group 2
                 testout.lo<-meldMC(c(mc1$Smc$Shi), 
                               c(mc2$Smc$Slo),
@@ -761,6 +952,8 @@ bpcp2samp<-function(time,status,group, testtime,
                               estimate2=list2$estimate) 
             } 
             if (dohi){
+                # Shi are upper Confidence distribution RVs
+                # Slo are lower CDRVs
                 # take Slo from group 1, and Shi from group 2
                 testout.hi<-meldMC(c(mc1$Smc$Slo), 
                               c(mc2$Smc$Shi),
@@ -774,7 +967,7 @@ bpcp2samp<-function(time,status,group, testtime,
             } 
             if (alt=="two.sided"){
                 testout<-testout.lo
-                # two.sided p-value is min(1,2*plower,2*pupper)
+                # two.sided p-value is min of twice the one-sided p-values (but not more than 1) 
                 testout$p.value<-min(1,2*testout.lo$p.value,
                                        2*testout.hi$p.value)
                 testout$conf.int<-c(testout.lo$conf.int[1],
@@ -790,8 +983,33 @@ bpcp2samp<-function(time,status,group, testtime,
 
     testout$method<-"Two-Sample Melded BPCP Test"
     if (midp) testout$method<- "Two-Sample Melded BPCP Test (mid-p version)"
-    names(testout$statistic)<-paste("S(",testtime,";group=",ug[1],")",sep="")
-    names(testout$parameter)<-paste("S(",testtime,";group=",ug[2],")",sep="")
+    if (ptype=="cdfratio"){
+      testout$statistic<- 1- testout$statistic
+      testout$parameter<- 1- testout$parameter
+      names(testout$statistic)<-paste("F(",testtime,";group=",ug[1],")",sep="")
+      names(testout$parameter)<-paste("F(",testtime,";group=",ug[2],")",sep="")
+    } else {
+      names(testout$statistic)<-paste("S(",testtime,";group=",ug[1],")",sep="")
+      names(testout$parameter)<-paste("S(",testtime,";group=",ug[2],")",sep="")
+    }
+    
+    # fix the one.minus PTypes
+    if (PType=="one.minus.ratio" | PType=="one.minus.cdfratio"){
+      testout$estimate<- 1-testout$estimate
+      names(testout$estimate)<-paste0("1-",names(testout$estimate))
+      testout$conf.int<- 1- testout$conf.int[2:1]
+      attr(testout$conf.int,"conf.level")<- conf.level
+      testout$null.value<- 1-testout$null.value
+      names(testout$null.value)<- paste0("Null effect (1-",names(testout$null.value),")")
+      testout$alternative<- switch.alt(alt)
+    } 
+    if (PType=="one.minus.ratio"){
+      testout$data.name<-paste("1-S(",testtime,";group=",ug[1],")/S(",testtime,";group=",ug[2],")",sep="")
+    }
+    if (PType=="one.minus.cdfratio"){
+      testout$data.name<-paste("1-F(",testtime,";group=",ug[1],")/F(",testtime,";group=",ug[2],")",sep="")
+    }
+    
     testout
 
 }
@@ -804,17 +1022,20 @@ bpcp2samp<-function(time,status,group, testtime,
 #bpcp2samp(c(1,3,5,4,9,2,7.1,9.5,8,10),rep(1,10),c(rep(1,6),rep(2,4)),5,control=bpcp2sampControl(method="mm.mc"),parmtype="oddsratio",midp=FALSE)
 #bpcp2samp(c(1,3,5,4,9,2,7.1,9.5,8,10),rep(1,10),c(rep(1,6),rep(2,4)),5,control=bpcp2sampControl(method="mc.mc"),parmtype="ratio",midp=FALSE)
 #bpcp2samp(c(1,3,5,4,9,2,7.1,9.5,8,10),rep(1,10),c(rep(1,6),rep(2,4)),5,control=bpcp2sampControl(method="mm.mc"),parmtype="ratio",midp=FALSE)
+#bpcp2samp(c(1,3,5,4,9,2,7.1,9.5,8,10),rep(1,10),c(rep(1,6),rep(2,4)),5,control=bpcp2sampControl(method="mc.mc"),parmtype="cdfratio",midp=FALSE)
+#bpcp2samp(c(1,3,5,4,9,2,7.1,9.5,8,10),rep(1,10),c(rep(1,6),rep(2,4)),5,control=bpcp2sampControl(method="mm.mc"),parmtype="cdfratio",midp=FALSE)
 
 
 
 
-
+# fixtdiff does asymptotic methods, with possible transformations
 fixtdiff<-function(time,status,group, testtime, 
     trans=c("identity","cloglog","log"),
     varpooled=TRUE, correct=FALSE, doall=FALSE){
 
  
-    if (class(group)=="factor"){
+    #if (class(group)=="factor"){
+    if (is(group,"factor")){
         # use order of levels, but only keep the levels with data 
         ug<- levels(group)
         ug<- ug[ug %in% group]
